import { IIterator } from '@lumino/algorithm';
import { ISignal } from '@lumino/signaling';
import { Tensorboard } from './tensorboard';
import { ServerConnection } from '@jupyterlab/services';
/**
 * A tensorboard manager.
 */
export declare class TensorboardManager implements Tensorboard.IManager {
    /**
     * Construct a new tensorboard manager.
     */
    constructor(options?: TensorboardManager.IOptions);
    /**
     * A signal emitted when the running tensorboards change.
     */
    get runningChanged(): ISignal<this, Tensorboard.IModel[]>;
    /**
     * Test whether the terminal manager is disposed.
     */
    get isDisposed(): boolean;
    /**
     * The server settings of the manager.
     */
    readonly serverSettings: ServerConnection.ISettings;
    /**
     * Dispose of the resources used by the manager.
     */
    dispose(): void;
    /**
     * Test whether the manager is ready.
     */
    get isReady(): boolean;
    /**
     * A promise that fulfills when the manager is ready.
     */
    get ready(): Promise<void>;
    /**
     * Create an iterator over the most recent running Tensorboards.
     *
     * @returns A new iterator over the running tensorboards.
     */
    running(): IIterator<Tensorboard.IModel>;
    /**
     * Create a new tensorboard.
     *
     * @param logdir - The logdir used to create a new tensorboard.
     *
     * @param options - The options used to connect to the tensorboard.
     *
     * @returns A promise that resolves with the tensorboard instance.
     */
    startNew(logdir: string, refreshInterval?: number, options?: Tensorboard.IOptions): Promise<Tensorboard.ITensorboard>;
    /**
     * Shut down a tensorboard by name.
     */
    shutdown(name: string): Promise<void>;
    /**
     * Shut down all tensorboards.
     *
     * @returns A promise that resolves when all of the tensorboards are shut down.
     */
    shutdownAll(): Promise<void>;
    /**
     * Force a refresh of the running tensorboards.
     *
     * @returns A promise that with the list of running tensorboards.
     */
    refreshRunning(): Promise<void>;
    /**
     * Handle a tensorboard terminating.
     */
    private _onTerminated;
    /**
     * Handle a tensorboard starting.
     */
    private _onStarted;
    /**
     * Refresh the running tensorboards.
     */
    private _refreshRunning;
    /**
     * Get a set of options to pass.
     */
    private _getOptions;
    private _models;
    private _tensorboards;
    private _isDisposed;
    private _isReady;
    private _readyPromise;
    private _refreshTimer;
    private _runningChanged;
}
/**
 * The namespace for TensorboardManager statics.
 */
export declare namespace TensorboardManager {
    /**
     * The options used to initialize a tensorboard manager.
     */
    interface IOptions {
        /**
         * The server settings used by the manager.
         */
        serverSettings?: ServerConnection.ISettings;
    }
}
