import { IIterator } from '@lumino/algorithm';
import { IDisposable } from '@lumino/disposable';
import { JSONObject } from '@lumino/coreutils';
import { Signal, ISignal } from '@lumino/signaling';
import { ServerConnection } from '@jupyterlab/services';
/**
 * The namespace for Tensorboard statics.
 */
export declare namespace Tensorboard {
    /**
     * An interface for a tensorboard.
     */
    interface ITensorboard extends IDisposable {
        /**
         * A signal emitted when the tensorboard is shut down.
         */
        terminated: ISignal<ITensorboard, void>;
        /**
         * The model associated with the tensorboard.
         */
        readonly model: IModel;
        /**
         * Get the name of the tensorboard.
         */
        readonly name: string;
        /**
         * The server settings for the tensorboard.
         */
        readonly serverSettings: ServerConnection.ISettings;
        /**
         * Shut down the tensorboard.
         */
        shutdown(): Promise<void>;
    }
    /**
     * Start a new tensorboard.
     *
     * @param options - The tensorboard options to use.
     *
     * @returns A promise that resolves with the tensorboard instance.
     */
    function startNew(logdir: string, refreshInterval: number, options?: IOptions): Promise<ITensorboard>;
    /**
     * List the running tensorboards.
     *
     * @param settings - The server settings to use.
     *
     * @returns A promise that resolves with the list of running tensorboard models.
     */
    function listRunning(settings?: ServerConnection.ISettings): Promise<IModel[]>;
    /**
     * Shut down a tensorboard by name.
     *
     * @param name - The name of the target tensorboard.
     *
     * @param settings - The server settings to use.
     *
     * @returns A promise that resolves when the tensorboard is shut down.
     */
    function shutdown(name: string, settings?: ServerConnection.ISettings): Promise<void>;
    /**
     * Shut down all tensorboard.
     *
     * @returns A promise that resolves when all of the tensorboards are shut down.
     */
    function shutdownAll(settings?: ServerConnection.ISettings): Promise<void>;
    /**
     * Get tensorboard's url
     */
    function getUrl(name: string, settings?: ServerConnection.ISettings): string;
    /**
     * The options for intializing a tensorboard object.
     */
    interface IOptions {
        /**
         * The server settings for the tensorboard.
         */
        serverSettings?: ServerConnection.ISettings;
    }
    /**
     * The server model for a tensorboard.
     */
    interface IModel extends JSONObject {
        /**
         * The name of the tensorboard.
         */
        readonly name: string;
        /**
         * The logdir Path of the tensorboard.
         */
        readonly logdir: string;
        /**
         * The reload interval of the tensorboard.
         */
        readonly reload_interval?: number;
        /**
         * The last reload time of the tensorboard.
         */
        readonly reload_time: string;
    }
    /**
     * The interface for a tensorboard manager.
     *
     * The manager is respoonsible for maintaining the state of running
     * tensorboard.
     */
    interface IManager extends IDisposable {
        readonly serverSettings: ServerConnection.ISettings;
        runningChanged: ISignal<this, IModel[]>;
        running(): IIterator<IModel>;
        startNew(logdir: string, refreshInterval: number, options?: IOptions): Promise<ITensorboard>;
        shutdown(name: string): Promise<void>;
        shutdownAll(): Promise<void>;
        refreshRunning(): Promise<void>;
    }
}
export declare class DefaultTensorboard implements Tensorboard.ITensorboard {
    /**
     * Construct a new tensorboard.
     */
    constructor(name: string, logdir: string, lastReload: string, reloadInterval: number | undefined, options?: Tensorboard.IOptions);
    /**
     * Get the name of the tensorboard.
     */
    get name(): string;
    /**
     * Get the model for the tensorboard.
     */
    get model(): Tensorboard.IModel;
    /**
     * A signal emitted when the tensorboard is shut down.
     */
    get terminated(): Signal<this, void>;
    /**
     * Test whether the tensorbaord is disposed.
     */
    get isDisposed(): boolean;
    /**
     * Dispose of the resources held by the tensorboard.
     */
    dispose(): void;
    /**
     * The server settings for the tensorboard.
     */
    readonly serverSettings: ServerConnection.ISettings;
    /**
     * Shut down the tensorboard.
     */
    shutdown(): Promise<void>;
    private _isDisposed;
    private _url;
    private _name;
    private _logdir;
    private _lastReload;
    private _reloadInterval;
    private _terminated;
}
/**
 * The static namespace for `DefaultTensorboard`.
 */
export declare namespace DefaultTensorboard {
    /**
     * Start a new tensorboard.
     *
     * @param options - The tensorboard options to use.
     *
     * @returns A promise that resolves with the tensorboard instance.
     */
    function startNew(logdir: string, refreshInterval: number, options?: Tensorboard.IOptions): Promise<Tensorboard.ITensorboard>;
    /**
     * List the running tensorboards.
     *
     * @param settings - The server settings to use.
     *
     * @returns A promise that resolves with the list of running tensorboard models.
     */
    function listRunning(settings?: ServerConnection.ISettings): Promise<Tensorboard.IModel[]>;
    /**
     * Shut down a tensorboard by name.
     *
     * @param name - Then name of the target tensorboard.
     *
     * @param settings - The server settings to use.
     *
     * @returns A promise that resolves when the tensorboard is shut down.
     */
    function shutdown(name: string, settings?: ServerConnection.ISettings): Promise<void>;
    /**
     * Shut down all tensorboards.
     *
     * @param settings - The server settings to use.
     *
     * @returns A promise that resolves when all the tensorboards are shut down.
     */
    function shutdownAll(settings?: ServerConnection.ISettings): Promise<void>;
    /**
     * According tensorboard's name to get tensorboard's url.
     */
    function getUrl(name: string, settings?: ServerConnection.ISettings): string;
}
