import { each, map, toArray } from '@lumino/algorithm';
import { URLExt } from '@jupyterlab/coreutils';
import { Signal } from '@lumino/signaling';
import { ServerConnection } from '@jupyterlab/services';
/**
 * The url for the tensorboard service. tensorboard
 * service provided by jupyter_tensorboard.
 * ref: https://github.com/lspvic/jupyter_tensorboard
 * Maybe rewrite the jupyter_tensorboard service by myself.
 */
const TENSORBOARD_SERVICE_URL = 'api/tensorboard_pro';
const TENSORBOARD_URL = 'tensorboard_pro';
/**
 * The namespace for Tensorboard statics.
 */
export var Tensorboard;
(function (Tensorboard) {
    /**
     * Start a new tensorboard.
     *
     * @param options - The tensorboard options to use.
     *
     * @returns A promise that resolves with the tensorboard instance.
     */
    function startNew(logdir, refreshInterval, options) {
        return DefaultTensorboard.startNew(logdir, refreshInterval, options);
    }
    Tensorboard.startNew = startNew;
    /**
     * List the running tensorboards.
     *
     * @param settings - The server settings to use.
     *
     * @returns A promise that resolves with the list of running tensorboard models.
     */
    function listRunning(settings) {
        return DefaultTensorboard.listRunning(settings);
    }
    Tensorboard.listRunning = listRunning;
    /**
     * Shut down a tensorboard by name.
     *
     * @param name - The name of the target tensorboard.
     *
     * @param settings - The server settings to use.
     *
     * @returns A promise that resolves when the tensorboard is shut down.
     */
    function shutdown(name, settings) {
        return DefaultTensorboard.shutdown(name, settings);
    }
    Tensorboard.shutdown = shutdown;
    /**
     * Shut down all tensorboard.
     *
     * @returns A promise that resolves when all of the tensorboards are shut down.
     */
    function shutdownAll(settings) {
        return DefaultTensorboard.shutdownAll(settings);
    }
    Tensorboard.shutdownAll = shutdownAll;
    /**
     * Get tensorboard's url
     */
    function getUrl(name, settings) {
        return DefaultTensorboard.getUrl(name, settings);
    }
    Tensorboard.getUrl = getUrl;
})(Tensorboard || (Tensorboard = {}));
export class DefaultTensorboard {
    /**
     * Construct a new tensorboard.
     */
    constructor(name, logdir, lastReload, reloadInterval, options = {}) {
        this._isDisposed = false;
        this._terminated = new Signal(this);
        this._name = name;
        this._logdir = logdir;
        this._lastReload = lastReload;
        this._reloadInterval = reloadInterval;
        this.serverSettings = options.serverSettings || ServerConnection.makeSettings();
        this._url = Private.getTensorboardInstanceUrl(this.serverSettings.baseUrl, this._name);
    }
    /**
     * Get the name of the tensorboard.
     */
    get name() {
        return this._name;
    }
    /**
     * Get the model for the tensorboard.
     */
    get model() {
        return {
            name: this._name,
            logdir: this._logdir,
            reload_time: this._lastReload,
            reload_interval: this._reloadInterval
        };
    }
    /**
     * A signal emitted when the tensorboard is shut down.
     */
    get terminated() {
        return this._terminated;
    }
    /**
     * Test whether the tensorbaord is disposed.
     */
    get isDisposed() {
        return this._isDisposed;
    }
    /**
     * Dispose of the resources held by the tensorboard.
     */
    dispose() {
        if (this._isDisposed) {
            return;
        }
        this.terminated.emit(void 0);
        this._isDisposed = true;
        delete Private.running[this._url];
        Signal.clearData(this);
    }
    /**
     * Shut down the tensorboard.
     */
    shutdown() {
        const { name, serverSettings } = this;
        return DefaultTensorboard.shutdown(name, serverSettings);
    }
}
/**
 * The static namespace for `DefaultTensorboard`.
 */
(function (DefaultTensorboard) {
    /**
     * Start a new tensorboard.
     *
     * @param options - The tensorboard options to use.
     *
     * @returns A promise that resolves with the tensorboard instance.
     */
    function startNew(logdir, refreshInterval, options = {}) {
        const serverSettings = options.serverSettings || ServerConnection.makeSettings();
        const url = Private.getServiceUrl(serverSettings.baseUrl);
        // ServerConnection won't automaticy add this header when the body in not none.
        const header = new Headers({ 'Content-Type': 'application/json' });
        const data = JSON.stringify({ logdir: logdir, reload_interval: refreshInterval });
        const init = { method: 'POST', headers: header, body: data };
        console.log(`tensorboard start: url: ${url}, init: `, init, 'data:', data, ', serverSettings:', serverSettings);
        return ServerConnection.makeRequest(url, init, serverSettings)
            .then(response => {
            if (response.status !== 200) {
                throw new ServerConnection.ResponseError(response);
            }
            return response.json();
        })
            .then((data) => {
            const name = data.name;
            const logdir = data.logdir;
            const lastReload = data.reload_time;
            const reloadInterval = data.reload_interval;
            return new DefaultTensorboard(name, logdir, lastReload, reloadInterval, Object.assign(Object.assign({}, options), { serverSettings }));
        });
    }
    DefaultTensorboard.startNew = startNew;
    /**
     * List the running tensorboards.
     *
     * @param settings - The server settings to use.
     *
     * @returns A promise that resolves with the list of running tensorboard models.
     */
    function listRunning(settings) {
        settings = settings || ServerConnection.makeSettings();
        const service_url = Private.getServiceUrl(settings.baseUrl);
        const instance_url = Private.getTensorboardInstanceRootUrl(settings.baseUrl);
        return ServerConnection.makeRequest(service_url, {}, settings)
            .then(response => {
            if (response.status !== 200) {
                throw new ServerConnection.ResponseError(response);
            }
            return response.json();
        })
            .then((data) => {
            if (!Array.isArray(data)) {
                throw new Error('Invalid tensorboard data');
            }
            // Update the local data store.
            const urls = toArray(map(data, item => {
                return URLExt.join(instance_url, item.name);
            }));
            each(Object.keys(Private.running), runningUrl => {
                if (urls.indexOf(runningUrl) === -1) {
                    const tensorboard = Private.running[runningUrl];
                    tensorboard.dispose();
                }
            });
            return data;
        });
    }
    DefaultTensorboard.listRunning = listRunning;
    /**
     * Shut down a tensorboard by name.
     *
     * @param name - Then name of the target tensorboard.
     *
     * @param settings - The server settings to use.
     *
     * @returns A promise that resolves when the tensorboard is shut down.
     */
    function shutdown(name, settings) {
        settings = settings || ServerConnection.makeSettings();
        const url = Private.getTensorboardUrl(settings.baseUrl, name);
        const init = { method: 'DELETE' };
        return ServerConnection.makeRequest(url, init, settings).then(response => {
            if (response.status === 404) {
                return response.json().then(data => {
                    console.warn(data['message']);
                    Private.killTensorboard(url);
                });
            }
            if (response.status !== 204) {
                throw new ServerConnection.ResponseError(response);
            }
            Private.killTensorboard(url);
        });
    }
    DefaultTensorboard.shutdown = shutdown;
    /**
     * Shut down all tensorboards.
     *
     * @param settings - The server settings to use.
     *
     * @returns A promise that resolves when all the tensorboards are shut down.
     */
    function shutdownAll(settings) {
        settings = settings || ServerConnection.makeSettings();
        return listRunning(settings).then(running => {
            each(running, s => {
                shutdown(s.name, settings);
            });
        });
    }
    DefaultTensorboard.shutdownAll = shutdownAll;
    /**
     * According tensorboard's name to get tensorboard's url.
     */
    function getUrl(name, settings) {
        settings = settings || ServerConnection.makeSettings();
        return Private.getTensorboardInstanceUrl(settings.baseUrl, name);
    }
    DefaultTensorboard.getUrl = getUrl;
})(DefaultTensorboard || (DefaultTensorboard = {}));
/**
 * A namespace for private data.
 */
var Private;
(function (Private) {
    /**
     * A mapping of running tensorboards by url.
     */
    Private.running = Object.create(null);
    /**
     * Get the url for a tensorboard.
     */
    function getTensorboardUrl(baseUrl, name) {
        return URLExt.join(baseUrl, TENSORBOARD_SERVICE_URL, name);
    }
    Private.getTensorboardUrl = getTensorboardUrl;
    /**
     * Get the base url.
     */
    function getServiceUrl(baseUrl) {
        return URLExt.join(baseUrl, TENSORBOARD_SERVICE_URL);
    }
    Private.getServiceUrl = getServiceUrl;
    /**
     * Kill tensorboard by url.
     */
    function killTensorboard(url) {
        // Update the local data store.
        if (Private.running[url]) {
            const tensorboard = Private.running[url];
            tensorboard.dispose();
        }
    }
    Private.killTensorboard = killTensorboard;
    function getTensorboardInstanceRootUrl(baseUrl) {
        return URLExt.join(baseUrl, TENSORBOARD_URL);
    }
    Private.getTensorboardInstanceRootUrl = getTensorboardInstanceRootUrl;
    function getTensorboardInstanceUrl(baseUrl, name) {
        return URLExt.join(baseUrl, TENSORBOARD_URL, name);
    }
    Private.getTensorboardInstanceUrl = getTensorboardInstanceUrl;
})(Private || (Private = {}));
